@@ -40,19 +40,50 @@ func detectConfigFileType(path, def string) string {
 	}
 }
 
-func IsConfederationMember(g *Global, p *Neighbor) bool {
-	if p.Config.PeerAs != g.Config.As {
-		for _, member := range g.Confederation.Config.MemberAsList {
-			if member == p.Config.PeerAs {
-				return true
-			}
+func (b *BgpConfigSet) getPeerGroup(n string) (*PeerGroup, error) {
+	if n == "" {
+		return nil, nil
+	}
+	for _, pg := range b.PeerGroups {
+		if n == pg.Config.PeerGroupName {
+			return &pg, nil
+		}
+	}
+	return nil, fmt.Errorf("no such peer-group: %s", n)
+}
+
+func (d *DynamicNeighbor) validate(b *BgpConfigSet) error {
+	if d.Config.PeerGroup == "" {
+		return fmt.Errorf("dynamic neighbor requires the peer group config")
+	}
+
+	if _, err := b.getPeerGroup(d.Config.PeerGroup); err != nil {
+		return err
+	}
+	if _, _, err := net.ParseCIDR(d.Config.Prefix); err != nil {
+		return fmt.Errorf("invalid dynamic neighbor prefix %s", d.Config.Prefix)
+	}
+	return nil
+}
+
+func (n *Neighbor) IsConfederationMember(g *Global) bool {
+	for _, member := range g.Confederation.Config.MemberAsList {
+		if member == n.Config.PeerAs {
+			return true
 		}
 	}
 	return false
 }
 
-func IsEBGPPeer(g *Global, p *Neighbor) bool {
-	return p.Config.PeerAs != g.Config.As
+func (n *Neighbor) IsConfederation(g *Global) bool {
+	if n.Config.PeerAs == g.Config.As {
+		return true
+	}
+	return n.IsConfederationMember(g)
+}
+
+func (n *Neighbor) IsEBGPPeer(g *Global) bool {
+	return n.Config.PeerAs != g.Config.As
 }
 
 type AfiSafis []AfiSafi
@@ -66,17 +97,16 @@ func (c AfiSafis) ToRfList() ([]bgp.RouteFamily, error) {
 }
 
 func CreateRfMap(p *Neighbor) map[bgp.RouteFamily]bgp.BGPAddPathMode {
-	rfs, _ := AfiSafis(p.AfiSafis).ToRfList()
-	mode := bgp.BGP_ADD_PATH_NONE
-	if p.AddPaths.Config.Receive {
-		mode |= bgp.BGP_ADD_PATH_RECEIVE
-	}
-	if p.AddPaths.Config.SendMax > 0 {
-		mode |= bgp.BGP_ADD_PATH_SEND
-	}
 	rfMap := make(map[bgp.RouteFamily]bgp.BGPAddPathMode)
-	for _, rf := range rfs {
-		rfMap[rf] = mode
+	for _, af := range p.AfiSafis {
+		mode := bgp.BGP_ADD_PATH_NONE
+		if af.AddPaths.State.Receive {
+			mode |= bgp.BGP_ADD_PATH_RECEIVE
+		}
+		if af.AddPaths.State.SendMax > 0 {
+			mode |= bgp.BGP_ADD_PATH_SEND
+		}
+		rfMap[af.State.Family] = mode
 	}
 	return rfMap
 }
@@ -154,3 +184,12 @@ func ExtractNeighborAddress(c *Neighbor) (string, error) {
 	}
 	return addr, nil
 }
+
+func (n *Neighbor) IsAddPathReceiveEnabled(family bgp.RouteFamily) bool {
+	for _, af := range n.AfiSafis {
+		if af.State.Family == family {
+			return af.AddPaths.State.Receive
+		}
+	}
+	return false
+}
