@@ -31,6 +31,29 @@ import (
 	"github.com/osrg/gobgp/table"
 )
 
+// used in showRoute() to determine the width of each column
+var (
+	columnWidthPrefix  = 20
+	columnWidthNextHop = 20
+	columnWidthAsPath  = 20
+	columnWidthLabel   = 10
+)
+
+func updateColumnWidth(nlri, nexthop, aspath, label string) {
+	if prefixLen := len(nlri); columnWidthPrefix < prefixLen {
+		columnWidthPrefix = prefixLen
+	}
+	if columnWidthNextHop < len(nexthop) {
+		columnWidthNextHop = len(nexthop)
+	}
+	if columnWidthAsPath < len(aspath) {
+		columnWidthAsPath = len(aspath)
+	}
+	if columnWidthLabel < len(label) {
+		columnWidthLabel = len(label)
+	}
+}
+
 func getNeighbors(vrf string) (neighbors, error) {
 	if vrf != "" {
 		n, err := client.ListNeighborByVRF(vrf)
@@ -51,13 +74,6 @@ func getNeighbors(vrf string) (neighbors, error) {
 	return neighbors(n), err
 }
 
-func getNeighbor(name string, enableAdvertised bool) (*config.Neighbor, error) {
-	if net.ParseIP(name) == nil {
-		name = ""
-	}
-	return client.GetNeighbor(name, enableAdvertised)
-}
-
 func getASN(p *config.Neighbor) string {
 	asn := "*"
 	if p.State.PeerAs > 0 {
@@ -155,7 +171,7 @@ func showNeighbors(vrf string) error {
 }
 
 func showNeighbor(args []string) error {
-	p, e := getNeighbor(args[0], true)
+	p, e := client.GetNeighbor(args[0], true)
 	if e != nil {
 		return e
 	}
@@ -407,146 +423,137 @@ type AsPathFormat struct {
 	separator string
 }
 
-func ShowRoute(pathList []*table.Path, showAge, showBest, showLabel, isMonitor, printHeader bool, showIdentifier bgp.BGPAddPathMode) {
-
-	var pathStrs [][]interface{}
-	maxPrefixLen := 20
-	maxNexthopLen := 20
-	maxAsPathLen := 20
-	maxLabelLen := 10
-
-	now := time.Now()
-	for idx, p := range pathList {
-		nexthop := "fictitious"
-		if n := p.GetNexthop(); n != nil {
-			nexthop = p.GetNexthop().String()
-		}
-		aspathstr := p.GetAsString()
-
-		s := []string{}
-		for _, a := range p.GetPathAttrs() {
-			switch a.GetType() {
-			case bgp.BGP_ATTR_TYPE_NEXT_HOP, bgp.BGP_ATTR_TYPE_MP_REACH_NLRI, bgp.BGP_ATTR_TYPE_AS_PATH, bgp.BGP_ATTR_TYPE_AS4_PATH:
-				continue
-			default:
-				s = append(s, a.String())
-			}
-		}
-		pattrstr := fmt.Sprint(s)
-
-		if maxNexthopLen < len(nexthop) {
-			maxNexthopLen = len(nexthop)
-		}
-
-		if maxAsPathLen < len(aspathstr) {
-			maxAsPathLen = len(aspathstr)
-		}
-
-		best := ""
-		if p.IsStale() {
-			best += "S"
-		}
-		switch p.ValidationStatus() {
-		case config.RPKI_VALIDATION_RESULT_TYPE_NOT_FOUND:
-			best += "N"
-		case config.RPKI_VALIDATION_RESULT_TYPE_VALID:
-			best += "V"
-		case config.RPKI_VALIDATION_RESULT_TYPE_INVALID:
-			best += "I"
-		}
-		if showBest {
-			if idx == 0 && !p.IsNexthopInvalid {
-				best += "*>"
-			} else {
-				best += "* "
-			}
-		}
-		nlri := p.GetNlri()
-		if maxPrefixLen < len(nlri.String()) {
-			maxPrefixLen = len(nlri.String())
-		}
-
-		if isMonitor {
-			title := "ROUTE"
-			if p.IsWithdraw {
-				title = "DELROUTE"
-			}
-			if showIdentifier != bgp.BGP_ADD_PATH_NONE {
-				pathStrs = append(pathStrs, []interface{}{title, nlri.PathIdentifier(), nlri, nexthop, aspathstr, pattrstr})
-			} else {
-				pathStrs = append(pathStrs, []interface{}{title, nlri, nexthop, aspathstr, pattrstr})
-			}
+func getPathSymbolString(p *table.Path, idx int, showBest bool) string {
+	symbols := ""
+	if p.IsStale() {
+		symbols += "S"
+	}
+	switch p.ValidationStatus() {
+	case config.RPKI_VALIDATION_RESULT_TYPE_NOT_FOUND:
+		symbols += "N"
+	case config.RPKI_VALIDATION_RESULT_TYPE_VALID:
+		symbols += "V"
+	case config.RPKI_VALIDATION_RESULT_TYPE_INVALID:
+		symbols += "I"
+	}
+	if showBest {
+		if idx == 0 && !p.IsNexthopInvalid {
+			symbols += "*>"
 		} else {
-			args := []interface{}{best}
-			switch showIdentifier {
-			case bgp.BGP_ADD_PATH_RECEIVE:
-				args = append(args, fmt.Sprint(nlri.PathIdentifier()))
-			case bgp.BGP_ADD_PATH_SEND:
-				args = append(args, fmt.Sprint(nlri.PathLocalIdentifier()))
+			symbols += "* "
+		}
+	}
+	return symbols
+}
+
+func getPathAttributeString(p *table.Path) string {
+	s := make([]string, 0)
+	for _, a := range p.GetPathAttrs() {
+		switch a.GetType() {
+		case bgp.BGP_ATTR_TYPE_NEXT_HOP, bgp.BGP_ATTR_TYPE_MP_REACH_NLRI, bgp.BGP_ATTR_TYPE_AS_PATH, bgp.BGP_ATTR_TYPE_AS4_PATH:
+			continue
+		default:
+			s = append(s, a.String())
+		}
+	}
+	switch n := p.GetNlri().(type) {
+	case *bgp.EVPNNLRI:
+		// We print non route key fields like path attributes.
+		switch route := n.RouteTypeData.(type) {
+		case *bgp.EVPNMacIPAdvertisementRoute:
+			s = append(s, fmt.Sprintf("[ESI: %s]", route.ESI.String()))
+		case *bgp.EVPNIPPrefixRoute:
+			s = append(s, fmt.Sprintf("[ESI: %s]", route.ESI.String()))
+			if route.GWIPAddress != nil {
+				s = append(s, fmt.Sprintf("[GW: %s]", route.GWIPAddress.String()))
 			}
-			args = append(args, nlri)
-			if showLabel {
-				label := ""
-				switch nlri.(type) {
-				case *bgp.LabeledIPAddrPrefix:
-					label = nlri.(*bgp.LabeledIPAddrPrefix).Labels.String()
-				case *bgp.LabeledIPv6AddrPrefix:
-					label = nlri.(*bgp.LabeledIPv6AddrPrefix).Labels.String()
-				case *bgp.LabeledVPNIPAddrPrefix:
-					label = nlri.(*bgp.LabeledVPNIPAddrPrefix).Labels.String()
-				case *bgp.LabeledVPNIPv6AddrPrefix:
-					label = nlri.(*bgp.LabeledVPNIPv6AddrPrefix).Labels.String()
-				}
-				if maxLabelLen < len(label) {
-					maxLabelLen = len(label)
-				}
-				args = append(args, label)
-			}
-			args = append(args, []interface{}{nexthop, aspathstr}...)
-			if showAge {
-				args = append(args, formatTimedelta(int64(now.Sub(p.GetTimestamp()).Seconds())))
-			}
-			args = append(args, pattrstr)
-			pathStrs = append(pathStrs, args)
+		}
+	}
+	return fmt.Sprint(s)
+}
+
+func makeShowRouteArgs(p *table.Path, idx int, now time.Time, showAge, showBest, showLabel bool, showIdentifier bgp.BGPAddPathMode) []interface{} {
+	nlri := p.GetNlri()
+
+	// Path Symbols (e.g. "*>")
+	args := []interface{}{getPathSymbolString(p, idx, showBest)}
+
+	// Path Identifier
+	switch showIdentifier {
+	case bgp.BGP_ADD_PATH_RECEIVE:
+		args = append(args, fmt.Sprint(nlri.PathIdentifier()))
+	case bgp.BGP_ADD_PATH_SEND:
+		args = append(args, fmt.Sprint(nlri.PathLocalIdentifier()))
+	}
+
+	// NLRI
+	args = append(args, nlri)
+
+	// Label
+	label := ""
+	if showLabel {
+		label = p.GetLabelString()
+		args = append(args, label)
+	}
+
+	// Next Hop
+	nexthop := "fictitious"
+	if n := p.GetNexthop(); n != nil {
+		nexthop = p.GetNexthop().String()
+	}
+	args = append(args, nexthop)
+
+	// AS_PATH
+	aspathstr := p.GetAsString()
+	args = append(args, aspathstr)
+
+	// Age
+	if showAge {
+		args = append(args, formatTimedelta(int64(now.Sub(p.GetTimestamp()).Seconds())))
+	}
+
+	// Path Attributes
+	pattrstr := getPathAttributeString(p)
+	args = append(args, pattrstr)
+
+	updateColumnWidth(nlri.String(), nexthop, aspathstr, label)
+
+	return args
+}
+
+func showRoute(destinationList [][]*table.Path, showAge, showBest, showLabel bool, showIdentifier bgp.BGPAddPathMode) {
+	var pathStrs [][]interface{}
+	now := time.Now()
+	for _, pathList := range destinationList {
+		for idx, p := range pathList {
+			pathStrs = append(pathStrs, makeShowRouteArgs(p, idx, now, showAge, showBest, showLabel, showIdentifier))
 		}
 	}
 
+	headers := make([]interface{}, 0)
 	var format string
-	if isMonitor {
-		format = "[%s] %d:%s via %s aspath [%s] attrs %s\n"
-	} else {
-		format = fmt.Sprintf("%%-3s")
-		if showIdentifier != bgp.BGP_ADD_PATH_NONE {
-			format += "%-3s "
-		}
-		format += fmt.Sprintf("%%-%ds ", maxPrefixLen)
-		if showLabel {
-			format += fmt.Sprintf("%%-%ds ", maxLabelLen)
-		}
-		format += fmt.Sprintf("%%-%ds %%-%ds ", maxNexthopLen, maxAsPathLen)
-		if showAge {
-			format += "%-10s "
-		}
-		format += "%-s\n"
+	headers = append(headers, "") // Symbols
+	format = fmt.Sprintf("%%-3s")
+	if showIdentifier != bgp.BGP_ADD_PATH_NONE {
+		headers = append(headers, "ID")
+		format += "%-3s "
 	}
-
-	if printHeader {
-		args := []interface{}{""}
-		if showIdentifier != bgp.BGP_ADD_PATH_NONE {
-			args = append(args, "ID")
-		}
-		args = append(args, "Network")
-		if showLabel {
-			args = append(args, "Labels")
-		}
-		args = append(args, []interface{}{"Next Hop", "AS_PATH"}...)
-		if showAge {
-			args = append(args, "Age")
-		}
-		args = append(args, "Attrs")
-		fmt.Printf(format, args...)
+	headers = append(headers, "Network")
+	format += fmt.Sprintf("%%-%ds ", columnWidthPrefix)
+	if showLabel {
+		headers = append(headers, "Labels")
+		format += fmt.Sprintf("%%-%ds ", columnWidthLabel)
 	}
+	headers = append(headers, "Next Hop", "AS_PATH")
+	format += fmt.Sprintf("%%-%ds %%-%ds ", columnWidthNextHop, columnWidthAsPath)
+	if showAge {
+		headers = append(headers, "Age")
+		format += "%-10s "
+	}
+	headers = append(headers, "Attrs")
+	format += "%-s\n"
 
+	fmt.Printf(format, headers...)
 	for _, pathStr := range pathStrs {
 		fmt.Printf(format, pathStr...)
 	}
@@ -568,10 +575,16 @@ func checkOriginAsWasNotShown(p *table.Path, shownAs map[uint32]struct{}) bool {
 	return true
 }
 
-func ShowValidationInfo(p *table.Path) {
+func showValidationInfo(p *table.Path, shownAs map[uint32]struct{}) error {
+	asPath := p.GetAsPath().Value
+	if len(asPath) == 0 {
+		return fmt.Errorf("The path to %s was locally generated.\n", p.GetNlri().String())
+	} else if !checkOriginAsWasNotShown(p, shownAs) {
+		return nil
+	}
+
 	status := p.Validation().Status
 	reason := p.Validation().Reason
-	asPath := p.GetAsPath().Value
 	aslist := asPath[len(asPath)-1].(*bgp.As4PathParam).AS
 	origin := aslist[len(aslist)-1]
 
@@ -615,6 +628,8 @@ func ShowValidationInfo(p *table.Path) {
 	printVRPs(p.Validation().UnmatchedAs)
 	fmt.Println("  Unmatched Length VRPs: ")
 	printVRPs(p.Validation().UnmatchedLength)
+
+	return nil
 }
 
 func showRibInfo(r, name string) error {
@@ -692,7 +707,7 @@ func showNeighborRib(r string, name string, args []string) error {
 		return err
 	}
 	switch family {
-	case bgp.RF_IPv4_MPLS, bgp.RF_IPv6_MPLS, bgp.RF_IPv4_VPN, bgp.RF_IPv6_VPN:
+	case bgp.RF_IPv4_MPLS, bgp.RF_IPv6_MPLS, bgp.RF_IPv4_VPN, bgp.RF_IPv6_VPN, bgp.RF_EVPN:
 		showLabel = true
 	}
 
@@ -749,7 +764,7 @@ func showNeighborRib(r string, name string, args []string) error {
 	switch r {
 	case CMD_LOCAL, CMD_ADJ_IN, CMD_ACCEPTED, CMD_REJECTED, CMD_ADJ_OUT:
 		if rib.Info("").NumDestination == 0 {
-			peer, err := getNeighbor(name, false)
+			peer, err := client.GetNeighbor(name, false)
 			if err != nil {
 				return err
 			}
@@ -765,51 +780,49 @@ func showNeighborRib(r string, name string, args []string) error {
 		return nil
 	}
 
-	shownAs := make(map[uint32]struct{})
-	counter := 0
-	for _, d := range rib.GetSortedDestinations() {
-		if validationTarget != "" && d.GetNlri().String() != validationTarget {
-			continue
+	if validationTarget != "" {
+		// show RPKI validation info
+		d := rib.GetDestination(validationTarget)
+		if d == nil {
+			fmt.Println("Network not in table")
+			return nil
 		}
-		var ps []*table.Path
-		if r == CMD_ACCEPTED || r == CMD_REJECTED {
-			for _, p := range d.GetAllKnownPathList() {
-				switch r {
-				case CMD_ACCEPTED:
+		shownAs := make(map[uint32]struct{})
+		for _, p := range d.GetAllKnownPathList() {
+			if err := showValidationInfo(p, shownAs); err != nil {
+				return err
+			}
+		}
+	} else {
+		// show RIB
+		var ds [][]*table.Path
+		for _, d := range rib.GetSortedDestinations() {
+			var ps []*table.Path
+			switch r {
+			case CMD_ACCEPTED:
+				for _, p := range d.GetAllKnownPathList() {
 					if p.Filtered("") > table.POLICY_DIRECTION_NONE {
 						continue
 					}
-				case CMD_REJECTED:
+					ps = append(ps, p)
+				}
+			case CMD_REJECTED:
+				for _, p := range d.GetAllKnownPathList() {
 					if p.Filtered("") == table.POLICY_DIRECTION_NONE {
 						continue
 					}
+					ps = append(ps, p)
 				}
-				ps = append(ps, p)
+			default:
+				ps = d.GetAllKnownPathList()
 			}
+			ds = append(ds, ps)
+		}
+		if len(ds) > 0 {
+			showRoute(ds, showAge, showBest, showLabel, showIdentifier)
 		} else {
-			ps = d.GetAllKnownPathList()
+			fmt.Println("Network not in table")
 		}
-		showHeader := false
-		if counter == 0 {
-			showHeader = true
-		}
-		if validationTarget != "" {
-			for _, p := range ps {
-				asPath := p.GetAsPath().Value
-				if len(asPath) == 0 {
-					fmt.Printf("The path to %s was locally generated.\n", p.GetNlri().String())
-				} else if checkOriginAsWasNotShown(p, shownAs) {
-					ShowValidationInfo(p)
-				}
-			}
-		} else {
-			ShowRoute(ps, showAge, showBest, showLabel, false, showHeader, showIdentifier)
-		}
-		counter++
-	}
-
-	if counter == 0 {
-		fmt.Println("Network not in table")
 	}
 	return nil
 }
@@ -955,13 +968,13 @@ func modNeighborPolicy(remoteIP, policyType, cmdType string, args []string) erro
 }
 
 func modNeighbor(cmdType string, args []string) error {
-	m := extractReserved(args, []string{"interface", "as", "vrf", "route-reflector-client", "route-server-client", "allow-own-as", "remove-private-as", "replace-peer-as"})
-	usage := fmt.Sprintf("usage: gobgp neighbor %s [<neighbor-address>| interface <neighbor-interface>]", cmdType)
+	m := extractReserved(args, []string{"interface", "as", "family", "vrf", "route-reflector-client", "route-server-client", "allow-own-as", "remove-private-as", "replace-peer-as"})
+	usage := fmt.Sprintf("usage: gobgp neighbor %s [ <neighbor-address> | interface <neighbor-interface> ]", cmdType)
 	if cmdType == CMD_ADD {
-		usage += " as <VALUE> [ vrf <vrf-name> | route-reflector-client [<cluster-id>] | route-server-client | allow-own-as <num> | remove-private-as (all|replace) | replace-peer-as ]"
+		usage += " as <VALUE> [ family <address-families-list> | vrf <vrf-name> | route-reflector-client [<cluster-id>] | route-server-client | allow-own-as <num> | remove-private-as (all|replace) | replace-peer-as ]"
 	}
 
-	if (len(m[""]) != 1 && len(m["interface"]) != 1) || len(m["as"]) > 1 || len(m["vrf"]) > 1 || len(m["route-reflector-client"]) > 1 || len(m["allow-own-as"]) > 1 || len(m["remove-private-as"]) > 1 {
+	if (len(m[""]) != 1 && len(m["interface"]) != 1) || len(m["as"]) > 1 || len(m["family"]) > 1 || len(m["vrf"]) > 1 || len(m["route-reflector-client"]) > 1 || len(m["allow-own-as"]) > 1 || len(m["remove-private-as"]) > 1 {
 		return fmt.Errorf("%s", usage)
 	}
 	unnumbered := len(m["interface"]) > 0
@@ -983,6 +996,15 @@ func modNeighbor(cmdType string, args []string) error {
 			peer.Config.NeighborAddress = m[""][0]
 			peer.State.NeighborAddress = m[""][0]
 		}
+		if len(m["family"]) == 1 {
+			for _, family := range strings.Split(m["family"][0], ",") {
+				afiSafiName := config.AfiSafiType(family)
+				if afiSafiName.ToInt() == -1 {
+					return nil, fmt.Errorf("invalid family value: %s", family)
+				}
+				peer.AfiSafis = append(peer.AfiSafis, config.AfiSafi{Config: config.AfiSafiConfig{AfiSafiName: afiSafiName}})
+			}
+		}
 		if len(m["vrf"]) == 1 {
 			peer.Config.Vrf = m["vrf"][0]
 		}
@@ -1076,7 +1098,7 @@ func NewNeighborCmd() *cobra.Command {
 						}
 					}
 					if addr == "" {
-						peer, err := getNeighbor(args[len(args)-1], false)
+						peer, err := client.GetNeighbor(args[len(args)-1], false)
 						if err != nil {
 							exitWithError(err)
 						}
@@ -1107,7 +1129,7 @@ func NewNeighborCmd() *cobra.Command {
 	policyCmd := &cobra.Command{
 		Use: CMD_POLICY,
 		Run: func(cmd *cobra.Command, args []string) {
-			peer, err := getNeighbor(args[0], false)
+			peer, err := client.GetNeighbor(args[0], false)
 			if err != nil {
 				exitWithError(err)
 			}
@@ -1124,7 +1146,7 @@ func NewNeighborCmd() *cobra.Command {
 		cmd := &cobra.Command{
 			Use: v,
 			Run: func(cmd *cobra.Command, args []string) {
-				peer, err := getNeighbor(args[0], false)
+				peer, err := client.GetNeighbor(args[0], false)
 				if err != nil {
 					exitWithError(err)
 				}
@@ -1140,7 +1162,7 @@ func NewNeighborCmd() *cobra.Command {
 			subcmd := &cobra.Command{
 				Use: w,
 				Run: func(subcmd *cobra.Command, args []string) {
-					peer, err := getNeighbor(args[len(args)-1], false)
+					peer, err := client.GetNeighbor(args[len(args)-1], false)
 					if err != nil {
 						exitWithError(err)
 					}
