@@ -559,6 +559,26 @@ func (h *FSMHandler) active() (bgp.FSMState, FsmStateReason) {
 	}
 }
 
+func capAddPathFromConfig(pConf *config.Neighbor) bgp.ParameterCapabilityInterface {
+	tuples := make([]*bgp.CapAddPathTuple, 0, len(pConf.AfiSafis))
+	for _, af := range pConf.AfiSafis {
+		var mode bgp.BGPAddPathMode
+		if af.AddPaths.State.Receive {
+			mode |= bgp.BGP_ADD_PATH_RECEIVE
+		}
+		if af.AddPaths.State.SendMax > 0 {
+			mode |= bgp.BGP_ADD_PATH_SEND
+		}
+		if mode > 0 {
+			tuples = append(tuples, bgp.NewCapAddPathTuple(af.State.Family, mode))
+		}
+	}
+	if len(tuples) == 0 {
+		return nil
+	}
+	return bgp.NewCapAddPath(tuples)
+}
+
 func capabilitiesFromConfig(pConf *config.Neighbor) []bgp.ParameterCapabilityInterface {
 	caps := make([]bgp.ParameterCapabilityInterface, 0, 4)
 	caps = append(caps, bgp.NewCapRouteRefresh())
@@ -595,9 +615,9 @@ func capabilitiesFromConfig(pConf *config.Neighbor) []bgp.ParameterCapabilityInt
 				}
 			}
 		}
-		time := c.RestartTime
+		restartTime := c.RestartTime
 		notification := c.NotificationEnabled
-		caps = append(caps, bgp.NewCapGracefulRestart(restarting, notification, time, tuples))
+		caps = append(caps, bgp.NewCapGracefulRestart(restarting, notification, restartTime, tuples))
 		if c.LongLivedEnabled {
 			caps = append(caps, bgp.NewCapLongLivedGracefulRestart(ltuples))
 		}
@@ -614,23 +634,12 @@ func capabilitiesFromConfig(pConf *config.Neighbor) []bgp.ParameterCapabilityInt
 			tuple := bgp.NewCapExtendedNexthopTuple(family, bgp.AFI_IP6)
 			tuples = append(tuples, tuple)
 		}
-		cap := bgp.NewCapExtendedNexthop(tuples)
-		caps = append(caps, cap)
+		caps = append(caps, bgp.NewCapExtendedNexthop(tuples))
 	}
 
-	var mode bgp.BGPAddPathMode
-	if pConf.AddPaths.Config.Receive {
-		mode |= bgp.BGP_ADD_PATH_RECEIVE
-	}
-	if pConf.AddPaths.Config.SendMax > 0 {
-		mode |= bgp.BGP_ADD_PATH_SEND
-	}
-	if uint8(mode) > 0 {
-		items := make([]*bgp.CapAddPathTuple, 0, len(pConf.AfiSafis))
-		for _, af := range pConf.AfiSafis {
-			items = append(items, bgp.NewCapAddPathTuple(af.State.Family, mode))
-		}
-		caps = append(caps, bgp.NewCapAddPath(items))
+	// ADD-PATH Capability
+	if c := capAddPathFromConfig(pConf); c != nil {
+		caps = append(caps, capAddPathFromConfig(pConf))
 	}
 
 	return caps
@@ -750,7 +759,7 @@ func (h *FSMHandler) handlingError(m *bgp.BGPMessage, e error, useRevisedError b
 					"Topic": "Peer",
 					"Key":   h.fsm.pConf.State.NeighborAddress,
 					"State": h.fsm.state.String(),
-				}).Warn("Error occured during AFI/SAFI disabling")
+				}).Warn("Error occurred during AFI/SAFI disabling")
 			} else {
 				n := h.afiSafiDisable(*rf)
 				log.WithFields(log.Fields{
@@ -847,17 +856,17 @@ func (h *FSMHandler) recvMessageWithError() (*FsmMsg, error) {
 				fmsg.MsgType = FSM_MSG_ROUTE_REFRESH
 			case bgp.BGP_MSG_UPDATE:
 				body := m.Body.(*bgp.BGPUpdate)
-				confedCheck := !config.IsConfederationMember(h.fsm.gConf, h.fsm.pConf) && config.IsEBGPPeer(h.fsm.gConf, h.fsm.pConf)
+				isEBGP := h.fsm.pConf.IsEBGPPeer(h.fsm.gConf)
+				isConfed := h.fsm.pConf.IsConfederationMember(h.fsm.gConf)
 
 				fmsg.payload = make([]byte, len(headerBuf)+len(bodyBuf))
 				copy(fmsg.payload, headerBuf)
 				copy(fmsg.payload[len(headerBuf):], bodyBuf)
 
-				ok, err := bgp.ValidateUpdateMsg(body, h.fsm.rfMap, confedCheck)
+				ok, err := bgp.ValidateUpdateMsg(body, h.fsm.rfMap, isEBGP, isConfed)
 				if !ok {
 					handling = h.handlingError(m, err, useRevisedError)
 				}
-
 				if handling == bgp.ERROR_HANDLING_SESSION_RESET {
 					log.WithFields(log.Fields{
 						"Topic": "Peer",
@@ -1090,7 +1099,7 @@ func (h *FSMHandler) opensent() (bgp.FSMState, FsmStateReason) {
 							"Topic": "Peer",
 							"Key":   fsm.pConf.State.NeighborAddress,
 							"State": fsm.state.String(),
-						}).Infof("skiped asn negotiation: peer-as: %d, peer-type: %s", peerAs, typ)
+						}).Infof("skipped asn negotiation: peer-as: %d, peer-type: %s", peerAs, typ)
 					} else {
 						fsm.pConf.State.PeerType = fsm.pConf.Config.PeerType
 					}
@@ -1436,7 +1445,7 @@ func (h *FSMHandler) sendMessageloop() error {
 			return nil
 		case o := <-h.outgoing.Out():
 			m := o.(*FsmOutgoingMsg)
-			for _, msg := range table.CreateUpdateMsgFromPaths(m.Paths) {
+			for _, msg := range table.CreateUpdateMsgFromPaths(m.Paths, h.fsm.marshallingOptions) {
 				if err := send(msg); err != nil {
 					return nil
 				}
